@@ -1,7 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 # -*- coding: utf-8 -*-
 
-import os
 import json
 import datetime
 import warnings
@@ -9,23 +8,22 @@ import copy
 from numbers import Integral
 from functools import lru_cache
 from collections import OrderedDict
-from collections.abc import Mapping
 from urllib import parse as urlparse
 
 from jsonschema import validators as mvalidators
 from jsonschema.exceptions import ValidationError
 
 import yaml
+import numpy as np
 
 from . import constants
 from . import generic_io
 from . import reference
 from . import treeutil
 from . import util
-from .extension import default_extensions
-from .compat.jsonschemacompat import JSONSCHEMA_LT_3
 from . import extension
-from .exceptions import AsdfDeprecationWarning
+from . import yamlutil
+from .exceptions import AsdfDeprecationWarning, AsdfWarning
 
 
 YAML_SCHEMA_METASCHEMA_ID = 'http://stsci.edu/schemas/yaml-schema/draft-01'
@@ -181,7 +179,7 @@ def validate_fill_default(validator, properties, instance, schema):
 
 
 FILL_DEFAULTS = util.HashableDict()
-for key in ('allOf', 'anyOf', 'oneOf', 'items'):
+for key in ('allOf', 'items'):
     FILL_DEFAULTS[key] = mvalidators.Draft4Validator.VALIDATORS[key]
 FILL_DEFAULTS['properties'] = validate_fill_default
 
@@ -201,101 +199,129 @@ def validate_remove_default(validator, properties, instance, schema):
 
 
 REMOVE_DEFAULTS = util.HashableDict()
-for key in ('allOf', 'anyOf', 'oneOf', 'items'):
+for key in ('allOf', 'items'):
     REMOVE_DEFAULTS[key] = mvalidators.Draft4Validator.VALIDATORS[key]
 REMOVE_DEFAULTS['properties'] = validate_remove_default
 
 
-@lru_cache()
-def _create_validator(validators=YAML_VALIDATORS):
-    meta_schema = load_schema(YAML_SCHEMA_METASCHEMA_ID, extension.get_default_resolver())
+class _ValidationContext:
+    """
+    Context that tracks (tree node, schema fragment) pairs that have
+    already been validated.
 
-    if JSONSCHEMA_LT_3:
-        base_cls = mvalidators.create(meta_schema=meta_schema, validators=validators)
-    else:
-        type_checker = mvalidators.Draft4Validator.TYPE_CHECKER.redefine(
-            'array',
-            lambda checker, instance: isinstance(instance, list) or isinstance(instance, tuple)
-        )
-        id_of = mvalidators.Draft4Validator.ID_OF
-        base_cls = mvalidators.create(
-            meta_schema=meta_schema,
-            validators=validators,
-            type_checker=type_checker,
-            id_of=id_of
-        )
+    Instances of this class are context managers that track
+    how many times they have been entered, and only reset themselves
+    when exiting the outermost context.
+    """
+    def __init__(self):
+        self._depth = 0
+        self._seen = set()
+
+    def add(self, instance, schema):
+        """
+        Inform the context that an instance has
+        been validated against a schema fragment.
+        """
+        self._seen.add(self._make_seen_key(instance, schema))
+
+    def seen(self, instance, schema):
+        """
+        Return True if an instance has already been
+        validated against a schema fragment.
+        """
+        return self._make_seen_key(instance, schema) in self._seen
+
+    def __enter__(self):
+        self._depth += 1
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self._depth -= 1
+
+        if self._depth == 0:
+            self._seen = set()
+
+    def _make_seen_key(self, instance, schema):
+        return (id(instance), id(schema))
+
+
+@lru_cache()
+def _create_validator(validators=YAML_VALIDATORS, visit_repeat_nodes=False):
+    meta_schema = _load_schema_cached(YAML_SCHEMA_METASCHEMA_ID, extension.get_default_resolver(), False, False)
+
+    type_checker = mvalidators.Draft4Validator.TYPE_CHECKER.redefine_many({
+        'array': lambda checker, instance: isinstance(instance, list) or isinstance(instance, tuple),
+        'integer': lambda checker, instance: not isinstance(instance, bool) and isinstance(instance, Integral),
+        'string': lambda checker, instance: isinstance(instance, (str, np.str_)),
+    })
+    id_of = mvalidators.Draft4Validator.ID_OF
+    base_cls = mvalidators.create(
+        meta_schema=meta_schema,
+        validators=validators,
+        type_checker=type_checker,
+        id_of=id_of
+    )
 
     class ASDFValidator(base_cls):
-        if JSONSCHEMA_LT_3:
-            DEFAULT_TYPES = base_cls.DEFAULT_TYPES.copy()
-            DEFAULT_TYPES['array'] = (list, tuple)
+        def __init__(self, *args, **kwargs):
+            super().__init__(*args, **kwargs)
+            self._context = _ValidationContext()
 
-        def iter_errors(self, instance, _schema=None, _seen=set()):
+        def iter_errors(self, instance, _schema=None):
             # We can't validate anything that looks like an external reference,
             # since we don't have the actual content, so we just have to defer
             # it for now.  If the user cares about complete validation, they
             # can call `AsdfFile.resolve_references`.
-            if id(instance) in _seen:
-                return
+            with self._context:
+                if _schema is None:
+                    schema = self.schema
+                else:
+                    schema = _schema
 
-            if _schema is None:
-                schema = self.schema
-            else:
-                schema = _schema
+                if self._context.seen(instance, schema):
+                    # We've already validated this instance against this schema,
+                    # no need to do it again.
+                    return
 
-            if ((isinstance(instance, dict) and '$ref' in instance) or
-                    isinstance(instance, reference.Reference)):
-                return
+                if not visit_repeat_nodes:
+                    self._context.add(instance, schema)
 
-            if _schema is None:
-                tag = getattr(instance, '_tag', None)
-                if tag is not None:
-                    schema_path = self.ctx.resolver(tag)
-                    if schema_path != tag:
-                        try:
-                            s = load_schema(schema_path, self.ctx.resolver)
-                        except FileNotFoundError:
-                            msg = "Unable to locate schema file for '{}': '{}'"
-                            warnings.warn(msg.format(tag, schema_path))
-                            s = {}
-                        if s:
-                            with self.resolver.in_scope(schema_path):
-                                for x in super(ASDFValidator, self).iter_errors(instance, s):
-                                    yield x
+                if ((isinstance(instance, dict) and '$ref' in instance) or
+                        isinstance(instance, reference.Reference)):
+                    return
 
-                if isinstance(instance, dict):
-                    new_seen = _seen | set([id(instance)])
-                    for val in instance.values():
-                        for x in self.iter_errors(val, _seen=new_seen):
-                            yield x
+                if _schema is None:
+                    tag = getattr(instance, '_tag', None)
+                    if tag is not None:
+                        schema_path = self.ctx.resolver(tag)
+                        if schema_path != tag:
+                            try:
+                                s = _load_schema_cached(schema_path, self.ctx.resolver, False, False)
+                            except FileNotFoundError:
+                                msg = "Unable to locate schema file for '{}': '{}'"
+                                warnings.warn(msg.format(tag, schema_path), AsdfWarning)
+                                s = {}
+                            if s:
+                                with self.resolver.in_scope(schema_path):
+                                    for x in super(ASDFValidator, self).iter_errors(instance, s):
+                                        yield x
 
-                elif isinstance(instance, list):
-                    new_seen = _seen | set([id(instance)])
-                    for val in instance:
-                        for x in self.iter_errors(val, _seen=new_seen):
-                            yield x
-            else:
-                for x in super(ASDFValidator, self).iter_errors(instance, _schema=schema):
-                    yield x
+                    if isinstance(instance, dict):
+                        for val in instance.values():
+                            for x in self.iter_errors(val):
+                                yield x
+
+                    elif isinstance(instance, list):
+                        for val in instance:
+                            for x in self.iter_errors(val):
+                                yield x
+                else:
+                    for x in super(ASDFValidator, self).iter_errors(instance, _schema=schema):
+                        yield x
 
     return ASDFValidator
 
 
-# We want to load mappings in schema as ordered dicts
-class OrderedLoader(_yaml_base_loader):
-    pass
-
-
-def construct_mapping(loader, node):
-    loader.flatten_mapping(node)
-    return OrderedDict(loader.construct_pairs(node))
-
-
-OrderedLoader.add_constructor(
-    yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
-    construct_mapping)
-
-
 @lru_cache()
 def _load_schema(url):
     with generic_io.get_file(url) as fd:
@@ -303,7 +329,7 @@ def _load_schema(url):
             json_data = fd.read().decode('utf-8')
             result = json.loads(json_data, object_pairs_hook=OrderedDict)
         else:
-            result = yaml.load(fd, Loader=OrderedLoader)
+            result = yaml.load(fd, Loader=yamlutil.AsdfLoader)
     return result, fd.uri
 
 
@@ -346,23 +372,12 @@ HARDCODED_SCHEMA = {
 
 @lru_cache()
 def load_custom_schema(url):
-    # Avoid circular import
-    from .tags.core import AsdfObject
-    custom = load_schema(url, resolve_local_refs=True)
-    core = load_schema(AsdfObject.yaml_tag)
-
-    def update(d, u):
-        for k, v in u.items():
-            # Respect the property ordering of the core schema
-            if k == 'propertyOrder' and k in d:
-                d[k] = u[k] + d[k]
-            elif isinstance(v, Mapping):
-                d[k] = update(d.get(k, {}), v)
-            else:
-                d[k] = v
-        return d
-
-    return update(custom, core)
+    warnings.warn(
+        "The 'load_custom_schema(...)' function is deprecated. Use"
+        "'load_schema' instead.",
+        AsdfDeprecationWarning
+    )
+    return load_schema(url, resolve_references=True)
 
 
 def load_schema(url, resolver=None, resolve_references=False,
@@ -389,7 +404,14 @@ def load_schema(url, resolver=None, resolve_references=False,
         within the same schema. This will automatically be handled when passing
         `resolve_references=True`, but it may be desirable in some cases to
         control local reference resolution separately.
+        This parameter is deprecated.
     """
+    if resolve_local_refs is True:
+        warnings.warn(
+            "The 'resolve_local_refs' parameter is deprecated.",
+            AsdfDeprecationWarning
+        )
+
     if resolver is None:
         # We can't just set this as the default in load_schema's definition
         # because invoking get_default_resolver at import time leads to a circular import.
@@ -455,7 +477,7 @@ def _load_schema_cached(url, resolver, resolve_references, resolve_local_refs):
 
 
 def get_validator(schema={}, ctx=None, validators=None, url_mapping=None,
-                  *args, **kwargs):
+                  *args, _visit_repeat_nodes=False, **kwargs):
     """
     Get a JSON schema validator object for the given schema.
 
@@ -478,6 +500,13 @@ def get_validator(schema={}, ctx=None, validators=None, url_mapping=None,
     url_mapping : resolver.Resolver, optional
         A resolver to convert remote URLs into local ones.
 
+    _visit_repeat_nodes : bool, optional
+        Force the validator to visit nodes that it has already
+        seen.  This flag is a temporary hack to support a specific
+        project that uses a custom validator to update a .fits file.
+        Setting `True` is discouraged and will lead to RecursionError
+        in trees containing reference cycles.
+
     Returns
     -------
     validator : jsonschema.Validator
@@ -497,7 +526,7 @@ def get_validator(schema={}, ctx=None, validators=None, url_mapping=None,
     # time of this writing, it was half of the runtime of the unit
     # test suite!!!).  Instead, we assume that the schemas are valid
     # through the running of the unit tests, not at run time.
-    cls = _create_validator(validators=validators)
+    cls = _create_validator(validators=validators, visit_repeat_nodes=_visit_repeat_nodes)
     validator = cls(schema, *args, **kwargs)
     validator.ctx = ctx
     return validator
@@ -524,7 +553,8 @@ def validate_large_literals(instance, reading=False):
         warnings.warn(
             "Invalid integer literal value {0} detected while reading file. "
             "The value has been read safely, but the file should be "
-            "fixed.".format(instance)
+            "fixed.".format(instance),
+            AsdfWarning
         )
 
 
@@ -603,40 +633,47 @@ def remove_defaults(instance, ctx):
     validate(instance, ctx, validators=REMOVE_DEFAULTS)
 
 
-def check_schema(schema):
+def check_schema(schema, validate_default=True):
     """
     Check a given schema to make sure it is valid YAML schema.
+
+    Parameters
+    ----------
+    schema : dict
+        The schema object, as returned by `load_schema`.
+    validate_default : bool, optional
+        Set to `True` to validate the content of the default
+        field against the schema.
     """
-    # We also want to validate the "default" values in the schema
-    # against the schema itself.  jsonschema as a library doesn't do
-    # this on its own.
-
-    def validate_default(validator, default, instance, schema):
-        if not validator.is_type(instance, 'object'):
-            return
-
-        if 'default' in instance:
-            with instance_validator.resolver.in_scope(scope):
-                for err in instance_validator.iter_errors(
-                        instance['default'], instance):
-                    yield err
-
-    VALIDATORS = util.HashableDict(
+    validators = util.HashableDict(
         mvalidators.Draft4Validator.VALIDATORS.copy())
-    VALIDATORS.update({
-        'default': validate_default
-    })
+
+    if validate_default:
+        # The jsonschema library doesn't validate defaults
+        # on its own.
+        instance_validator = get_validator(schema)
+        instance_scope = schema.get('id', '')
+
+        def _validate_default(validator, default, instance, schema):
+            if not validator.is_type(instance, 'object'):
+                return
+
+            if 'default' in instance:
+                with instance_validator.resolver.in_scope(instance_scope):
+                    for err in instance_validator.iter_errors(
+                            instance['default'], instance):
+                        yield err
+
+        validators.update({
+            'default': _validate_default
+        })
 
     meta_schema_id = schema.get('$schema', YAML_SCHEMA_METASCHEMA_ID)
-    meta_schema = load_schema(meta_schema_id, extension.get_default_resolver())
+    meta_schema = _load_schema_cached(meta_schema_id, extension.get_default_resolver(), False, False)
 
     resolver = _make_resolver(extension.get_default_resolver())
 
     cls = mvalidators.create(meta_schema=meta_schema,
-                             validators=VALIDATORS)
+                             validators=validators)
     validator = cls(meta_schema, resolver=resolver)
-
-    instance_validator = mvalidators.Draft4Validator(schema, resolver=resolver)
-    scope = schema.get('id', '')
-
     validator.validate(schema, _schema=meta_schema)
