@@ -10,14 +10,21 @@ from astropy.modeling import models
 
 import pytest
 
+from jsonschema.exceptions import ValidationError
+
 import asdf
 from asdf import treeutil
 from asdf import extension
 from asdf import resolver
 from asdf import schema
 from asdf import versioning
-from asdf.exceptions import AsdfDeprecationWarning
-from .helpers import assert_tree_match, assert_roundtrip_tree, display_warnings
+from asdf.exceptions import AsdfDeprecationWarning, AsdfWarning
+from .helpers import (
+    assert_tree_match,
+    assert_roundtrip_tree,
+    yaml_to_asdf,
+    assert_no_warnings,
+)
 
 
 def test_get_data_from_closed_file(tmpdir):
@@ -45,9 +52,8 @@ def test_no_warning_nan_array(tmpdir):
 
     tree = dict(array=np.array([1, 2, np.nan]))
 
-    with pytest.warns(None) as w:
+    with assert_no_warnings():
         assert_roundtrip_tree(tree, tmpdir)
-        assert len(w) == 0, display_warnings(w)
 
 
 def test_warning_deprecated_open(tmpdir):
@@ -82,6 +88,25 @@ def test_open_readonly(tmpdir):
             pass
 
 
+def test_open_validate_on_read(tmpdir):
+    content = """
+invalid_software: !core/software-1.0.0
+  name: Minesweeper
+  version: 3
+"""
+    buff = yaml_to_asdf(content)
+
+    with pytest.raises(ValidationError):
+        with asdf.open(buff, validate_on_read=True):
+            pass
+
+    buff.seek(0)
+
+    with asdf.open(buff, validate_on_read=False) as af:
+        assert af["invalid_software"]["name"] == "Minesweeper"
+        assert af["invalid_software"]["version"] == 3
+
+
 def test_atomic_write(tmpdir, small_tree):
     tmpfile = os.path.join(str(tmpdir), 'test.asdf')
 
@@ -282,16 +307,15 @@ def test_extension_version_check(installed, extension, warns):
     tree = {
         'history': {
             'extensions': [
-                asdf.tags.core.ExtensionMetadata('foo.extension.FooExtension',
-                    asdf.tags.core.Software(name='foo', version=extension)),
+                asdf.tags.core.ExtensionMetadata(extension_class='foo.extension.FooExtension',
+                    software=asdf.tags.core.Software(name='foo', version=extension)),
             ]
         }
     }
 
     if warns:
-        with pytest.warns(UserWarning) as w:
+        with pytest.warns(AsdfWarning, match="File 'test.asdf' was created with"):
             af._check_extensions(tree)
-        assert str(w[0].message).startswith("File 'test.asdf' was created with")
 
         with pytest.raises(RuntimeError) as err:
             af._check_extensions(tree, strict=True)
@@ -300,14 +324,80 @@ def test_extension_version_check(installed, extension, warns):
         af._check_extensions(tree)
 
 
-@pytest.mark.xfail(reason='Setting auto_inline option modifies AsdfFile state')
 def test_auto_inline(tmpdir):
-
     outfile = str(tmpdir.join('test.asdf'))
-    tree = dict(data=np.arange(6))
+    tree = {"small_array": np.arange(6), "large_array": np.arange(100)}
 
     # Use the same object for each write in order to make sure that there
     # aren't unanticipated side effects
+    with asdf.AsdfFile(tree) as af:
+        # By default blocks are written internal.
+        af.write_to(outfile)
+        assert len(list(af.blocks.inline_blocks)) == 0
+        assert len(list(af.blocks.internal_blocks)) == 2
+
+        af.write_to(outfile, auto_inline=10)
+        assert len(list(af.blocks.inline_blocks)) == 1
+        assert len(list(af.blocks.internal_blocks)) == 1
+
+        # The previous write modified the small array block's storage
+        # to inline, and a subsequent write should maintain that setting.
+        af.write_to(outfile)
+        assert len(list(af.blocks.inline_blocks)) == 1
+        assert len(list(af.blocks.internal_blocks)) == 1
+
+        af.write_to(outfile, auto_inline=7)
+        assert len(list(af.blocks.inline_blocks)) == 1
+        assert len(list(af.blocks.internal_blocks)) == 1
+
+        af.write_to(outfile, auto_inline=5)
+        assert len(list(af.blocks.inline_blocks)) == 0
+        assert len(list(af.blocks.internal_blocks)) == 2
+
+
+def test_auto_inline_masked_array(tmpdir):
+    outfile = str(tmpdir.join('test.asdf'))
+    arr = np.arange(6)
+    masked_arr = np.ma.masked_equal(arr, 3)
+    tree = {"masked_arr": masked_arr}
+
+    with asdf.AsdfFile(tree) as af:
+        af.write_to(outfile)
+        assert len(list(af.blocks.inline_blocks)) == 0
+        assert len(list(af.blocks.internal_blocks)) == 2
+
+        af.write_to(outfile, auto_inline=10)
+        assert len(list(af.blocks.inline_blocks)) == 2
+        assert len(list(af.blocks.internal_blocks)) == 0
+
+        af.write_to(outfile, auto_inline=5)
+        assert len(list(af.blocks.inline_blocks)) == 0
+        assert len(list(af.blocks.internal_blocks)) == 2
+
+
+def test_auto_inline_large_value(tmpdir):
+    outfile = str(tmpdir.join('test.asdf'))
+    arr = np.array([2**52 + 1, 1, 2, 3, 4, 5])
+    tree = {"array": arr}
+
+    with asdf.AsdfFile(tree) as af:
+        af.write_to(outfile)
+        assert len(list(af.blocks.inline_blocks)) == 0
+        assert len(list(af.blocks.internal_blocks)) == 1
+
+        with pytest.raises(ValidationError):
+            af.write_to(outfile, auto_inline=10)
+
+        af.write_to(outfile, auto_inline=5)
+        assert len(list(af.blocks.inline_blocks)) == 0
+        assert len(list(af.blocks.internal_blocks)) == 1
+
+
+def test_auto_inline_string_array(tmpdir):
+    outfile = str(tmpdir.join('test.asdf'))
+    arr = np.array(["peach", "plum", "apricot", "nectarine", "cherry", "pluot"])
+    tree = {"array": arr}
+
     with asdf.AsdfFile(tree) as af:
         af.write_to(outfile)
         assert len(list(af.blocks.inline_blocks)) == 0
@@ -317,101 +407,11 @@ def test_auto_inline(tmpdir):
         assert len(list(af.blocks.inline_blocks)) == 1
         assert len(list(af.blocks.internal_blocks)) == 0
 
-        af.write_to(outfile)
-        assert len(list(af.blocks.inline_blocks)) == 0
-        assert len(list(af.blocks.internal_blocks)) == 1
-
-        af.write_to(outfile, auto_inline=7)
-        assert len(list(af.blocks.inline_blocks)) == 1
-        assert len(list(af.blocks.internal_blocks)) == 0
-
         af.write_to(outfile, auto_inline=5)
         assert len(list(af.blocks.inline_blocks)) == 0
         assert len(list(af.blocks.internal_blocks)) == 1
 
 
-@pytest.mark.skip(reason='Until inline_threshold is added as a write option')
-def test_inline_threshold(tmpdir):
-
-    tree = {
-        'small': np.ones(10),
-        'large': np.ones(100)
-    }
-
-    with asdf.AsdfFile(tree) as af:
-        assert len(list(af.blocks.inline_blocks)) == 1
-        assert len(list(af.blocks.internal_blocks)) == 1
-
-    with asdf.AsdfFile(tree, inline_threshold=10) as af:
-        assert len(list(af.blocks.inline_blocks)) == 1
-        assert len(list(af.blocks.internal_blocks)) == 1
-
-    with asdf.AsdfFile(tree, inline_threshold=5) as af:
-        assert len(list(af.blocks.inline_blocks)) == 0
-        assert len(list(af.blocks.internal_blocks)) == 2
-
-    with asdf.AsdfFile(tree, inline_threshold=100) as af:
-        assert len(list(af.blocks.inline_blocks)) == 2
-        assert len(list(af.blocks.internal_blocks)) == 0
-
-
-@pytest.mark.skip(reason='Until inline_threshold is added as a write option')
-def test_inline_threshold_masked(tmpdir):
-
-    mask = np.random.randint(0, 1+1, 20)
-    masked_array = np.ma.masked_array(np.ones(20), mask=mask)
-
-    tree = {
-        'masked': masked_array
-    }
-
-    # Make sure that masked arrays aren't automatically inlined, even if they
-    # are small enough
-    with asdf.AsdfFile(tree) as af:
-        assert len(list(af.blocks.inline_blocks)) == 0
-        assert len(list(af.blocks.internal_blocks)) == 2
-
-    tree = {
-        'masked': masked_array,
-        'normal': np.random.random(20)
-    }
-
-    with asdf.AsdfFile(tree) as af:
-        assert len(list(af.blocks.inline_blocks)) == 1
-        assert len(list(af.blocks.internal_blocks)) == 2
-
-
-@pytest.mark.skip(reason='Until inline_threshold is added as a write option')
-def test_inline_threshold_override(tmpdir):
-
-    tmpfile = str(tmpdir.join('inline.asdf'))
-
-    tree = {
-        'small': np.ones(10),
-        'large': np.ones(100)
-    }
-
-    with asdf.AsdfFile(tree) as af:
-        af.set_array_storage(tree['small'], 'internal')
-        assert len(list(af.blocks.inline_blocks)) == 0
-        assert len(list(af.blocks.internal_blocks)) == 2
-
-    with asdf.AsdfFile(tree) as af:
-        af.set_array_storage(tree['large'], 'inline')
-        assert len(list(af.blocks.inline_blocks)) == 2
-        assert len(list(af.blocks.internal_blocks)) == 0
-
-    with asdf.AsdfFile(tree) as af:
-        af.write_to(tmpfile, all_array_storage='internal')
-        assert len(list(af.blocks.inline_blocks)) == 0
-        assert len(list(af.blocks.internal_blocks)) == 2
-
-    with asdf.AsdfFile(tree) as af:
-        af.write_to(tmpfile, all_array_storage='inline')
-        assert len(list(af.blocks.inline_blocks)) == 2
-        assert len(list(af.blocks.internal_blocks)) == 0
-
-
 def test_resolver_deprecations():
     for resolver_method in [
         resolver.default_resolver,
@@ -429,3 +429,123 @@ def test_get_default_resolver():
     result = resolver('tag:stsci.edu:asdf/core/ndarray-1.0.0')
 
     assert result.endswith("/schemas/stsci.edu/asdf/core/ndarray-1.0.0.yaml")
+
+
+def test_info_module(capsys, tmpdir):
+    tree = dict(
+        foo=42, bar="hello", baz=np.arange(20),
+        nested={"woo": "hoo", "yee": "haw"},
+        long_line="a" * 100
+    )
+    af = asdf.AsdfFile(tree)
+
+    def _assert_correct_info(node_or_path):
+        asdf.info(node_or_path)
+        captured = capsys.readouterr()
+        assert "foo" in captured.out
+        assert "bar" in captured.out
+        assert "baz" in captured.out
+
+    _assert_correct_info(af)
+    _assert_correct_info(af.tree)
+
+    tmpfile = str(tmpdir.join("written.asdf"))
+    af.write_to(tmpfile)
+    af.close()
+
+    _assert_correct_info(tmpfile)
+    _assert_correct_info(pathlib.Path(tmpfile))
+
+    for i in range(1, 10):
+        asdf.info(af, max_rows=i)
+        lines = capsys.readouterr().out.strip().split("\n")
+        assert len(lines) <= i
+
+    asdf.info(af, max_cols=80)
+    assert "(truncated)" in capsys.readouterr().out
+    asdf.info(af, max_cols=None)
+    captured = capsys.readouterr().out
+    assert "(truncated)" not in captured
+    assert "a" * 100 in captured
+
+    asdf.info(af, show_values=True)
+    assert "hello" in capsys.readouterr().out
+    asdf.info(af, show_values=False)
+    assert "hello" not in capsys.readouterr().out
+
+    tree = {
+        "foo": ["alpha", "bravo", "charlie", "delta", "eagle"]
+    }
+    af = asdf.AsdfFile(tree)
+    asdf.info(af, max_rows=(None,))
+    assert "alpha" not in capsys.readouterr().out
+    for i in range(1, 5):
+        asdf.info(af, max_rows=(None, i))
+        captured = capsys.readouterr()
+        for val in tree["foo"][0:i-1]:
+            assert val in captured.out
+        for val in tree["foo"][i-1:]:
+            assert val not in captured.out
+
+def test_info_asdf_file(capsys, tmpdir):
+    tree = dict(
+        foo=42, bar="hello", baz=np.arange(20),
+        nested={"woo": "hoo", "yee": "haw"},
+        long_line="a" * 100
+    )
+    af = asdf.AsdfFile(tree)
+    af.info()
+    captured = capsys.readouterr()
+    assert "foo" in captured.out
+    assert "bar" in captured.out
+    assert "baz" in captured.out
+
+
+def test_search():
+    tree = dict(foo=42, bar="hello", baz=np.arange(20))
+    af = asdf.AsdfFile(tree)
+
+    result = af.search("foo")
+    assert result.node == 42
+
+    result = af.search(type="ndarray")
+    assert (result.node == tree["baz"]).all()
+
+    result = af.search(value="hello")
+    assert result.node == "hello"
+
+
+def test_history_entries(tmpdir):
+    path = str(tmpdir.join("test.asdf"))
+    message = "Twas brillig, and the slithy toves"
+
+    af = asdf.AsdfFile()
+    af.add_history_entry(message)
+    af.write_to(path)
+    with asdf.open(path) as af:
+        assert af["history"]["entries"][0]["description"] == message
+
+    af = asdf.AsdfFile()
+    af.write_to(path)
+    with asdf.open(path) as af:
+        af.add_history_entry(message)
+        assert af["history"]["entries"][0]["description"] == message
+
+
+def test_array_access_after_file_close(tmpdir):
+    path = str(tmpdir.join("test.asdf"))
+    data = np.arange(10)
+    asdf.AsdfFile({"data": data}).write_to(path)
+
+    # Normally it's not possible to read the array after
+    # the file has been closed:
+    with asdf.open(path) as af:
+        tree = af.tree
+    with pytest.raises(OSError, match="ASDF file has already been closed"):
+        tree["data"][0]
+
+    # With memory mapping disabled and copying arrays enabled,
+    # the array data should still persist in memory after close:
+    with asdf.open(path, lazy_load=False, copy_arrays=True) as af:
+        tree = af.tree
+    assert_array_equal(tree["data"], data)
