@@ -4,7 +4,6 @@
 
 import copy
 import os
-import sys
 import pytest
 
 import numpy as np
@@ -14,19 +13,30 @@ astropy = pytest.importorskip('astropy')
 from astropy.io import fits
 from astropy.table import Table
 
+from jsonschema.exceptions import ValidationError
+
 import asdf
 from asdf import fits_embed
 from asdf import open as asdf_open
+from asdf.exceptions import AsdfWarning, AsdfConversionWarning
 
-from .helpers import assert_tree_match, display_warnings, get_test_data_path
+from .helpers import (
+    assert_tree_match,
+    get_test_data_path,
+    yaml_to_asdf,
+    assert_no_warnings,
+)
 
 
-def create_asdf_in_fits():
+TEST_DTYPES = ['<f8', '<f8', '<u4', '>u4', '<i4', '>i4']
+
+
+def create_asdf_in_fits(dtype):
     """Test fixture to create AsdfInFits object to use for testing"""
     hdulist = fits.HDUList()
-    hdulist.append(fits.ImageHDU(np.arange(512, dtype=np.float)))
-    hdulist.append(fits.ImageHDU(np.arange(512, dtype=np.float)))
-    hdulist.append(fits.ImageHDU(np.arange(512, dtype=np.float)))
+    hdulist.append(fits.ImageHDU(np.arange(512, dtype=dtype)))
+    hdulist.append(fits.ImageHDU(np.arange(512, dtype=dtype)))
+    hdulist.append(fits.ImageHDU(np.arange(512, dtype=dtype)))
 
     tree = {
         'model': {
@@ -50,15 +60,16 @@ def create_asdf_in_fits():
 # Testing backwards compatibility ensures that we can continue to read and
 # write files that use the old convention of ImageHDU to store the ASDF file.
 @pytest.mark.parametrize('backwards_compat', [False, True])
-def test_embed_asdf_in_fits_file(tmpdir, backwards_compat):
+@pytest.mark.parametrize('dtype', TEST_DTYPES)
+def test_embed_asdf_in_fits_file(tmpdir, backwards_compat, dtype):
     fits_testfile = str(tmpdir.join('test.fits'))
     asdf_testfile = str(tmpdir.join('test.asdf'))
 
     hdulist = fits.HDUList()
-    hdulist.append(fits.ImageHDU(np.arange(512, dtype=np.float), name='SCI'))
-    hdulist.append(fits.ImageHDU(np.arange(512, dtype=np.float), name='DQ'))
+    hdulist.append(fits.ImageHDU(np.arange(512, dtype=dtype), name='SCI'))
+    hdulist.append(fits.ImageHDU(np.arange(512, dtype=dtype), name='DQ'))
     # Test a name with underscores to make sure it works
-    hdulist.append(fits.ImageHDU(np.arange(512, dtype=np.float), name='WITH_UNDERSCORE'))
+    hdulist.append(fits.ImageHDU(np.arange(512, dtype=dtype), name='WITH_UNDERSCORE'))
 
     tree = {
         'model': {
@@ -83,14 +94,14 @@ def test_embed_asdf_in_fits_file(tmpdir, backwards_compat):
     with fits.open(fits_testfile) as hdulist2:
         assert len(hdulist2) == 4
         assert [x.name for x in hdulist2] == ['SCI', 'DQ', 'WITH_UNDERSCORE', 'ASDF']
-        assert_array_equal(hdulist2[0].data, np.arange(512, dtype=np.float))
+        assert_array_equal(hdulist2[0].data, np.arange(512, dtype=dtype))
         asdf_hdu = hdulist2['ASDF']
-        assert asdf_hdu.data.tostring().startswith(b'#ASDF')
+        assert asdf_hdu.data.tobytes().startswith(b'#ASDF')
         # When in backwards compatibility mode, the ASDF file will be contained
         # in an ImageHDU
         if backwards_compat:
             assert isinstance(asdf_hdu, fits.ImageHDU)
-            assert asdf_hdu.data.tostring().strip().endswith(b'...')
+            assert asdf_hdu.data.tobytes().strip().endswith(b'...')
         else:
             assert isinstance(asdf_hdu, fits.BinTableHDU)
 
@@ -104,9 +115,10 @@ def test_embed_asdf_in_fits_file(tmpdir, backwards_compat):
         assert_tree_match(tree, ff.tree)
 
 
-def test_embed_asdf_in_fits_file_anonymous_extensions(tmpdir):
+@pytest.mark.parametrize('dtype', TEST_DTYPES)
+def test_embed_asdf_in_fits_file_anonymous_extensions(tmpdir, dtype):
     # Write the AsdfInFits object out as a FITS file with ASDF extension
-    asdf_in_fits = create_asdf_in_fits()
+    asdf_in_fits = create_asdf_in_fits(dtype)
     asdf_in_fits.write_to(os.path.join(str(tmpdir), 'test.fits'))
 
     ff2 = asdf.AsdfFile(asdf_in_fits.tree)
@@ -117,7 +129,7 @@ def test_embed_asdf_in_fits_file_anonymous_extensions(tmpdir):
         assert [x.name for x in hdulist] == ['PRIMARY', '', '', 'ASDF']
         asdf_hdu = hdulist['ASDF']
         assert isinstance(asdf_hdu, fits.BinTableHDU)
-        assert asdf_hdu.data.tostring().startswith(b'#ASDF')
+        assert asdf_hdu.data.tobytes().startswith(b'#ASDF')
 
         with fits_embed.AsdfInFits.open(hdulist) as ff2:
             assert_tree_match(asdf_in_fits.tree, ff2.tree)
@@ -131,11 +143,12 @@ def test_embed_asdf_in_fits_file_anonymous_extensions(tmpdir):
 
 @pytest.mark.xfail(
     reason="In-place update for ASDF-in-FITS does not currently work")
-def test_update_in_place(tmpdir):
+@pytest.mark.parametrize('dtype', TEST_DTYPES)
+def test_update_in_place(tmpdir, dtype):
     tempfile = str(tmpdir.join('test.fits'))
 
     # Create a file and write it out
-    asdf_in_fits = create_asdf_in_fits()
+    asdf_in_fits = create_asdf_in_fits(dtype)
     asdf_in_fits.write_to(tempfile)
 
     # Open the file and add data so it needs to be updated
@@ -149,12 +162,13 @@ def test_update_in_place(tmpdir):
         assert_tree_match(ff.tree['model'], asdf_in_fits.tree['model'])
 
 
-def test_update_and_write_new(tmpdir):
+@pytest.mark.parametrize('dtype', TEST_DTYPES)
+def test_update_and_write_new(tmpdir, dtype):
     tempfile = str(tmpdir.join('test.fits'))
     newfile = str(tmpdir.join('new.fits'))
 
     # Create a file and write it out
-    asdf_in_fits = create_asdf_in_fits()
+    asdf_in_fits = create_asdf_in_fits(dtype)
     asdf_in_fits.write_to(tempfile)
 
     # Open the file and add data so it needs to be updated
@@ -170,32 +184,34 @@ def test_update_and_write_new(tmpdir):
 
 @pytest.mark.xfail(
     reason="ASDF HDU implementation does not currently reseek after writing")
-def test_access_hdu_data_after_write(tmpdir):
+@pytest.mark.parametrize('dtype', TEST_DTYPES)
+def test_access_hdu_data_after_write(tmpdir, dtype):
     # There is actually probably not a great reason to support this kind of
     # functionality, but I am adding a test here to record the failure for
     # posterity.
     tempfile = str(tmpdir.join('test.fits'))
 
-    asdf_in_fits = create_asdf_in_fits()
+    asdf_in_fits = create_asdf_in_fits(dtype)
     asdf_in_fits.write_to(tempfile)
     asdf_hdu = asdf_in_fits._hdulist['ASDF']
 
-    assert asdf_hdu.data.tostring().startswith('#ASDF')
+    assert asdf_hdu.data.tobytes().startswith('#ASDF')
 
 
-def test_create_in_tree_first(tmpdir):
+@pytest.mark.parametrize('dtype', TEST_DTYPES)
+def test_create_in_tree_first(tmpdir, dtype):
     tree = {
         'model': {
             'sci': {
-                'data': np.arange(512, dtype=np.float),
+                'data': np.arange(512, dtype=dtype),
                 'wcs': 'WCS info'
             },
             'dq': {
-                'data': np.arange(512, dtype=np.float),
+                'data': np.arange(512, dtype=dtype),
                 'wcs': 'WCS info'
             },
             'err': {
-                'data': np.arange(512, dtype=np.float),
+                'data': np.arange(512, dtype=dtype),
                 'wcs': 'WCS info'
             }
         }
@@ -215,13 +231,13 @@ def test_create_in_tree_first(tmpdir):
 
     with asdf.open(os.path.join(str(tmpdir), 'plain.asdf')) as ff:
         assert_array_equal(ff.tree['model']['sci']['data'],
-                           np.arange(512, dtype=np.float))
+                           np.arange(512, dtype=dtype))
 
     # This tests the changes that allow FITS files with ASDF extensions to be
     # opened directly by the top-level asdf.open API
     with asdf_open(tmpfile) as ff:
         assert_array_equal(ff.tree['model']['sci']['data'],
-                           np.arange(512, dtype=np.float))
+                           np.arange(512, dtype=dtype))
 
 def compare_asdfs(asdf0, asdf1):
     # Make sure the trees match
@@ -232,11 +248,13 @@ def compare_asdfs(asdf0, asdf1):
             asdf0.tree['model'][key]['data'],
             asdf1.tree['model'][key]['data'])
 
-def test_asdf_in_fits_open(tmpdir):
+
+@pytest.mark.parametrize('dtype', TEST_DTYPES)
+def test_asdf_in_fits_open(tmpdir, dtype):
     """Test the open method of AsdfInFits"""
     tmpfile = os.path.join(str(tmpdir), 'test.fits')
     # Write the AsdfInFits object out as a FITS file with ASDF extension
-    asdf_in_fits = create_asdf_in_fits()
+    asdf_in_fits = create_asdf_in_fits(dtype)
     asdf_in_fits.write_to(tmpfile)
 
     # Test opening the file directly from the URI
@@ -258,11 +276,13 @@ def test_asdf_in_fits_open(tmpdir):
         with fits_embed.AsdfInFits.open(hdulist) as ff:
             compare_asdfs(asdf_in_fits, ff)
 
-def test_asdf_open(tmpdir):
+
+@pytest.mark.parametrize('dtype', TEST_DTYPES)
+def test_asdf_open(tmpdir, dtype):
     """Test the top-level open method of the asdf module"""
     tmpfile = os.path.join(str(tmpdir), 'test.fits')
     # Write the AsdfInFits object out as a FITS file with ASDF extension
-    asdf_in_fits = create_asdf_in_fits()
+    asdf_in_fits = create_asdf_in_fits(dtype)
     asdf_in_fits.write_to(tmpfile)
 
     # Test opening the file directly from the URI
@@ -284,6 +304,33 @@ def test_asdf_open(tmpdir):
         with asdf_open(hdulist) as ff:
             compare_asdfs(asdf_in_fits, ff)
 
+def test_validate_on_read(tmpdir):
+    tmpfile = str(tmpdir.join('invalid.fits'))
+
+    content = """
+invalid_software: !core/software-1.0.0
+  name: Minesweeper
+  version: 3
+"""
+    buff = yaml_to_asdf(content)
+    hdul = fits.HDUList()
+    data = np.array(buff.getbuffer(), dtype=np.uint8)[None, :]
+    fmt = '{}B'.format(len(data[0]))
+    column = fits.Column(array=data, format=fmt, name='ASDF_METADATA')
+    hdu = fits.BinTableHDU.from_columns([column], name='ASDF')
+    hdul.append(hdu)
+    hdul.writeto(tmpfile)
+
+    for open_method in [asdf.open, fits_embed.AsdfInFits.open]:
+        with pytest.raises(ValidationError):
+            with open_method(tmpfile, validate_on_read=True):
+                pass
+
+        with open_method(tmpfile, validate_on_read=False) as af:
+            assert af["invalid_software"]["name"] == "Minesweeper"
+            assert af["invalid_software"]["version"] == 3
+
+
 def test_open_gzipped():
     testfile = get_test_data_path('asdf.fits.gz')
 
@@ -308,42 +355,29 @@ def test_bad_input(tmpdir):
     with pytest.raises(ValueError):
         asdf_open(text_file)
 
-@pytest.mark.skipif(sys.platform.startswith('win'),
-    reason='Avoid path manipulation on Windows')
 def test_version_mismatch_file():
-
     testfile = str(get_test_data_path('version_mismatch.fits'))
 
-    with pytest.warns(None) as w:
+    with pytest.warns(AsdfConversionWarning, match="tag:stsci.edu:asdf/core/complex"):
         with asdf.open(testfile,
                 ignore_version_mismatch=False) as fits_handle:
             assert fits_handle.tree['a'] == complex(0j)
-    # This is the warning that we expect from opening the FITS file
-    assert len(w) == 1, display_warnings(w)
-    assert str(w[0].message) == (
-        "'tag:stsci.edu:asdf/core/complex' with version 7.0.0 found in file "
-        "'{}', but latest supported version is 1.0.0".format(testfile))
 
     # Make sure warning does not occur when warning is ignored (default)
-    with pytest.warns(None) as w:
+    with assert_no_warnings(AsdfConversionWarning):
         with asdf.open(testfile) as fits_handle:
             assert fits_handle.tree['a'] == complex(0j)
-    assert len(w) == 0, display_warnings(w)
 
-    with pytest.warns(None) as w:
+    with pytest.warns(AsdfConversionWarning, match="tag:stsci.edu:asdf/core/complex"):
         with fits_embed.AsdfInFits.open(testfile,
                 ignore_version_mismatch=False) as fits_handle:
             assert fits_handle.tree['a'] == complex(0j)
-    assert len(w) == 1
-    assert str(w[0].message) == (
-        "'tag:stsci.edu:asdf/core/complex' with version 7.0.0 found in file "
-        "'{}', but latest supported version is 1.0.0".format(testfile))
 
     # Make sure warning does not occur when warning is ignored (default)
-    with pytest.warns(None) as w:
+    with assert_no_warnings(AsdfConversionWarning):
         with fits_embed.AsdfInFits.open(testfile) as fits_handle:
             assert fits_handle.tree['a'] == complex(0j)
-    assert len(w) == 0, display_warnings(w)
+
 
 def test_serialize_table(tmpdir):
     tmpfile = str(tmpdir.join('table.fits'))
@@ -366,29 +400,25 @@ def test_serialize_table(tmpdir):
 def test_extension_check():
     testfile = get_test_data_path('extension_check.fits')
 
-    with pytest.warns(None) as warnings:
-        with asdf.open(testfile) as ff:
+    with pytest.warns(AsdfWarning, match="was created with extension 'foo.bar.FooBar'"):
+        with asdf.open(testfile):
             pass
 
-    assert len(warnings) == 1, display_warnings(warnings)
-    assert ("was created with extension 'foo.bar.FooBar', which is not "
-        "currently installed (from package foo-1.2.3)") in str(warnings[0].message)
-
     # Make sure that suppressing the warning works as well
-    with pytest.warns(None) as warnings:
-        with asdf.open(testfile, ignore_missing_extensions=True) as ff:
+    with assert_no_warnings():
+        with asdf.open(testfile, ignore_missing_extensions=True):
             pass
 
-    assert len(warnings) == 0, display_warnings(warnings)
-
     with pytest.raises(RuntimeError):
-        with asdf.open(testfile, strict_extension_check=True) as ff:
+        with asdf.open(testfile, strict_extension_check=True):
             pass
 
-def test_verify_with_astropy(tmpdir):
+
+@pytest.mark.parametrize('dtype', TEST_DTYPES)
+def test_verify_with_astropy(tmpdir, dtype):
     tmpfile = str(tmpdir.join('asdf.fits'))
 
-    with create_asdf_in_fits() as aif:
+    with create_asdf_in_fits(dtype) as aif:
         aif.write_to(tmpfile)
 
     with fits.open(tmpfile) as hdu:
